@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2008	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -65,11 +65,13 @@ extern POOL_MEMORY_POOL *pool_memory_create(int blocksize);
 extern void pool_memory_delete(POOL_MEMORY_POOL *pool_memory, int reuse);
 extern char *pool_memory_strdup(POOL_MEMORY_POOL *pool_memory, const char *string);
 extern void *pool_memory_alloc_zero(POOL_MEMORY_POOL *pool_memory, unsigned int size);
+extern POOL_MEMORY_POOL *pool_memory_context_switch_to(POOL_MEMORY_POOL *pm);
 
 #define palloc(s) pool_memory_alloc(pool_memory, (s))
 #define pfree(p)  pool_memory_free(pool_memory, (p))
 #define repalloc(p, s)  pool_memory_realloc(pool_memory, (p), (s))
 #define pstrdup(s)  pool_memory_strdup(pool_memory, (s))
 #define palloc0(s) pool_memory_alloc_zero(pool_memory, (s))
+#define palloc0fast(s) pool_memory_alloc_zero(pool_memory, (s))	/* Added in 8.4 */
 
 #endif /* POOL_MEMORY_H */
