@@ -8,7 +8,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -192,12 +192,6 @@ pcp_authorize(char *username, char *password)
 	char encrypt_buf[(MD5_PASSWD_LEN+1)*2];
 	char md5[MD5_PASSWD_LEN+1];
 
-	if (strlen(username) >= MAX_USER_PASSWD_LEN)
-	{
-		fprintf(stderr, "ERROR: PCP authorization failed. username too long.\n");
-		return -1;
-	}
-
 	/* request salt */
 	pcp_write(pc, "M", 1);
 	wsize = htonl(sizeof(int));
@@ -717,14 +711,6 @@ pcp_process_info(int pid, int *array_size)
 			}
 			else if (strcmp(buf, "ProcessInfo") == 0)
 			{
-				if(process_info == NULL)
-				{
-					if (debug) fprintf(stderr, "DEBUG: invalid data.\"%s\"\n", buf);
-					free(buf);
-					errorcode = UNKNOWNERR;
-					return NULL;
-				}
-
 				process_info[offset].connection_info = &conn_info[offset];
 
 				index = (char *) memchr(buf, '\0', rsize) + 1;
@@ -737,7 +723,7 @@ pcp_process_info(int pid, int *array_size)
 			
 				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
-					strlcpy(process_info[offset].connection_info->user, index, SM_USER);
+					strcpy(process_info[offset].connection_info->user, index);
 			
 				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
@@ -822,22 +808,11 @@ pcp_systemdb_info(void)
 	}
 	if (debug) fprintf(stderr, "DEBUG: send: tos=\"S\", len=%d\n", ntohl(wsize));
 
-	while (1)
-	{
+	while (1) {
 		if (pcp_read(pc, &tos, 1))
-		{
-			if (buf != NULL)
-				free(buf);
 			return NULL;
-		}
-
 		if (pcp_read(pc, &rsize, sizeof(int)))
-		{
-			if (buf != NULL)
-				free(buf);
 			return NULL;
-		}
-
 		rsize = ntohl(rsize);
 		buf = (char *)malloc(rsize);
 		if (buf == NULL)
@@ -1371,14 +1346,6 @@ pcp_pool_status(int *array_size)
 			}
 			else if (strcmp(buf, "ProcessConfig") == 0)
 			{
-				if(status == NULL)
-				{
-					if (debug) fprintf(stderr, "DEBUG: invalid data.\"%s\"\n", buf);
-					free(buf);
-					errorcode = UNKNOWNERR;
-					return NULL;
-				}
-
 				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					strlcpy(status[offset].name, index, POOLCONFIG_MAXNAMELEN+1);
