@@ -517,7 +517,11 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 	socklen_t socklen;
 	int sts;
 
-#define CONNECT_TIMEOUT_MSEC 1000		/* specify select(2) timeout in millisecond */
+#define CONNECT_TIMEOUT_MSEC 1000		/* specify select(2) timeout in milliseconds */
+#define CONNECT_TIMEOUT_SEC CONNECT_TIMEOUT_MSEC/1000	/* seconds part */
+/* microseconds part */
+#define CONNECT_TIMEOUT_MICROSEC (CONNECT_TIMEOUT_SEC == 0?CONNECT_TIMEOUT_MSEC*1000:\
+								  CONNECT_TIMEOUT_MSEC*1000 - CONNECT_TIMEOUT_SEC*1000*1000)
 
 
 	fd = socket(AF_INET, SOCK_STREAM, 0);
@@ -565,7 +569,7 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 			return -1;
 		}
 
-		if (health_check_timer_expired)		/* has health check timer expired */
+		if (health_check_timer_expired && getpid() == mypid)		/* has health check timer expired */
 		{
 			pool_log("connect_inet_domain_socket_by_port: health check timer expired");
 			close(fd);
@@ -594,8 +598,8 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 				return -1;
 			}
 
-			timeout.tv_sec = 0;
-			timeout.tv_usec = CONNECT_TIMEOUT_MSEC * 1000;
+			timeout.tv_sec = CONNECT_TIMEOUT_SEC;
+			timeout.tv_usec = CONNECT_TIMEOUT_MICROSEC;
 			FD_ZERO(&rset);
 			FD_SET(fd, &rset);	
 			FD_ZERO(&wset);
@@ -607,9 +611,15 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 				/* select timeout */
 				if (retry)
 				{
-					pool_log("connect_inet_domain_socket: select() timedout. retrying...");
+					pool_log("connect_inet_domain_socket: select() timed out. retrying...");
 					continue;
 				}
+				else
+				{
+					pool_error("connect_inet_domain_socket: select() timed out");
+					close(fd);
+					return -1;
+				}
 			}
 			else if (sts > 0)
 			{
@@ -655,6 +665,9 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 					pool_log("connect_inet_domain_socket: select() interrupted. retrying...");
 					continue;
 				}
+				pool_log("connect_inet_domain_socket: select() interrupted");
+				close(fd);
+				return -1;
 			}
 		}
 		break;
