@@ -113,6 +113,7 @@ static int pool_hash_insert(POOL_QUERY_HASH *key, POOL_CACHEID *cacheid, bool up
 static uint32 create_hash_key(POOL_QUERY_HASH *key);
 static volatile POOL_HASH_ELEMENT *get_new_hash_element(void);
 static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element);
+static bool is_free_hash_element(void);
 static char *get_relation_without_alias(RangeVar *relation);
 
 /*
@@ -1203,6 +1204,7 @@ int pool_get_database_oid_from_dbname(char *dbname)
 	if (status != POOL_CONTINUE)
 	{    
 		pool_debug("pool_discard_oid_maps_by_db: Failed.");
+		free_select_result(res);
 		return 0;
 	}    
 
@@ -1766,7 +1768,7 @@ static int *pool_fsmm_clock_hand;
  */
 void pool_allocate_fsmm_clock_hand(void)
 {
-	pool_fsmm_clock_hand = pool_shared_memory_create(sizeof(pool_fsmm_clock_hand));
+	pool_fsmm_clock_hand = pool_shared_memory_create(sizeof(*pool_fsmm_clock_hand));
 	*pool_fsmm_clock_hand = 0;
 }
 
@@ -1828,7 +1830,7 @@ static POOL_CACHE_BLOCKID pool_get_block(size_t free_space)
 		return -1;
 	}
 
-	if (free_space < 0 || free_space > POOL_MAX_FREE_SPACE)
+	if (free_space > POOL_MAX_FREE_SPACE)
 	{
 		pool_error("pool_get_block: invalid free space %zd", free_space);
 		return -1;
@@ -1872,7 +1874,7 @@ static void pool_update_fsmm(POOL_CACHE_BLOCKID blockid, size_t free_space)
 		return;
 	}
 
-	if (blockid < 0 || blockid >= pool_get_memqcache_blocks())
+	if (blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_set_free_space: invalid block id %d", blockid);
 		return;
@@ -1956,6 +1958,16 @@ static POOL_CACHEID *pool_add_item_shmem_cache(POOL_QUERY_HASH *query_hash, char
 	 */
 	pool_init_cache_block(blockid);
 
+	/*
+	 * Make sure that we have at least one free hash element.
+	 */
+	while (!is_free_hash_element())
+	{
+		/* If not, reuse next victim block */
+		blockid = pool_reuse_block();
+		pool_init_cache_block(blockid);
+	}
+
 	/* Get block address on shmem */
 	p = block_address(blockid);
 	bh = (POOL_CACHE_BLOCK_HEADER *)p;
@@ -2251,7 +2263,7 @@ static int pool_delete_item_shmem_cache(POOL_CACHEID *cacheid)
 	pool_debug("pool_delete_item_shmem_cache: cacheid:%d itemid:%d",
 			   cacheid->blockid, cacheid->itemid);
 
-	if (cacheid->blockid < 0 || cacheid->blockid >= pool_get_memqcache_blocks())
+	if (cacheid->blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_delete_item_shmem_cache: invalid block id %d",
 				   cacheid->blockid);
@@ -2344,7 +2356,7 @@ static POOL_CACHE_ITEM_HEADER *pool_cache_item_header(POOL_CACHEID *cacheid)
 {
 	POOL_CACHE_BLOCK_HEADER *bh;
 
-	if (cacheid->blockid < 0 || cacheid->blockid >= pool_get_memqcache_blocks())
+	if (cacheid->blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_cache_item_header: invalid block id %d", cacheid->blockid);
 		return NULL;
@@ -2368,7 +2380,7 @@ static int pool_init_cache_block(POOL_CACHE_BLOCKID blockid)
 	char *p;
 	POOL_CACHE_BLOCK_HEADER *bh;
 
-	if (blockid < 0 || blockid >= pool_get_memqcache_blocks())
+	if (blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_init_cache_block: invalid block id %d", blockid);
 		return -1;
@@ -2593,6 +2605,7 @@ POOL_TEMP_QUERY_CACHE *pool_create_temp_query_cache(char *query)
 		if (!p->query)
 		{
 			pool_error("pool_create_temp_query_cache: strdup failed");
+			free(p);
 			return NULL;
 		}
 		p->buffer = pool_create_buffer();
@@ -3591,6 +3604,14 @@ static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element)
 	element->next = elm;
 }
 
+/*
+ * Return true if there's a free hash element.
+ */
+static bool is_free_hash_element(void)
+{
+	return hash_free->next != NULL;
+}
+
 /*
  * Returns shared memory cache stats.
  * Subsequent call to this function will break return value
