@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2013	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -68,7 +68,7 @@ static volatile sig_atomic_t restart_request = 0;
 static void establish_persistent_connection(void);
 static void discard_persistent_connection(void);
 static void check_replication_time_lag(void);
-static unsigned long long int text_to_lsn(char *text);
+static long text_to_lsn(char *text);
 static RETSIGTYPE my_signal_handler(int sig);
 static RETSIGTYPE reload_config_handler(int sig);
 static void reload_config(void);
@@ -164,7 +164,7 @@ static void establish_persistent_connection(void)
 											  bkinfo->backend_port,
 											  "postgres",
 											  pool_config->sr_check_user,
-											  pool_config->sr_check_password);
+											  pool_config->sr_check_password, true);
 			if (s)
 				slots[i] = s;
 			else
@@ -210,12 +210,6 @@ static void check_replication_time_lag(void)
 		return;
 	}
 
-	if (REAL_PRIMARY_NODE_ID < 0)
-	{
-		/* No need to check if there's no primary */
-		return;
-	}
-
 	/* Count healthy nodes */
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
@@ -324,7 +318,7 @@ static void check_replication_time_lag(void)
 /*
  * Convert logid/recoff style text to 64bit log location (LSN)
  */
-static unsigned long long int text_to_lsn(char *text)
+static long text_to_lsn(char *text)
 {
 /*
  * WAL segment size in bytes.  XXX We should fetch this from
