@@ -162,11 +162,13 @@ wd_create_send_socket(char * hostname, int port)
 	if ( setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *) &one, sizeof(one)) == -1 )
 	{
 		pool_error("wd_create_send_socket: setsockopt(TCP_NODELAY) failed. reason: %s", strerror(errno));
+		close(sock);
 		return -1;
 	}
 	if ( setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(one)) == -1 )
 	{
 		pool_error("wd_create_send_socket: setsockopt(SO_KEEPALIVE) failed. reason: %s", strerror(errno));
+		close(sock);
 		return -1;
 	}
 
@@ -440,8 +442,7 @@ wd_recv_packet(int sock, WdPacket * recv_pack)
 			if (read_size == len)
 			{
 
-				if ((ntohl(buf.packet_no) >= WD_INVALID) &&
-					(ntohl(buf.packet_no) <= WD_READY ))
+				if (ntohl(buf.packet_no) <= WD_READY)
 				{
 					ntoh_wd_packet(recv_pack,&buf);
 				}
